@@ -20,7 +20,7 @@ from typing import TYPE_CHECKING
 import bs4
 import magic
 import requests
-import stem.util.term
+import stem.util.term as stem_term
 
 from darc._compat import RobotFileParser
 from darc.const import (CHECK, CHECK_NG, LINK_BLACK_LIST, LINK_FALLBACK, LINK_WHITE_LIST,
@@ -35,17 +35,29 @@ if TYPE_CHECKING:
     from requests import Response
     from requests_futures.sessions import FuturesSession
 
-    from darc.link import Link
+    import darc.link as darc_link  # Link
 
 # Regular expression patterns to match all reasonable URLs.
-URL_PAT = [
+URL_PAT = {
+    # gfm.autolink.URL_RE (https://pythonhosted.org/py-gfm/_modules/gfm/autolink.html#AutolinkExtension)
+    'http': re.compile(r'(?i)\b(?P<url>(?:(?:ftp|https?|wss?|irc)://|www\d{0,3}[.])(?:[^\s()<>]+|'
+                       r'\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()'
+                       r'<>]+\)))*\)|[^\s`!()\[\]{};:' + r"'" + r'".,<>?«»“”‘’]))', re.ASCII),
+    # gfm.automail.MAIL_RE (https://pythonhosted.org/py-gfm/_modules/gfm/automail.html#AutomailExtension)
+    'mailto:': re.compile(r'(?i)\b(?P<url>(?:mailto:)?[a-z0-9._%+-]+@[a-z0-9.-]+\.[a-z]+)\b', re.ASCII),
+
+    # BTC
+    'bitcoin': re.compile(r'(?i)\b(?P<url>(?:(?:bitcoin|btc):)?[13][a-z0-9]{27,34})\b', re.ASCII),
+    # ETH
+    'ethereum': re.compile(r'(?i)\b(?P<url>(?:(?:ethereum|eth):)?(?:0x)?[0-9a-f]{40})\b', re.ASCII),
+
     # HTTP(S) and other *regular* URLs, e.g. WebSocket, IRC, etc.
-    re.compile(r'(?P<url>((https?|wss?|irc):)?(//)?\w+(\.\w+)+/?\S*)', re.UNICODE),
+    #re.compile(r'(?P<url>((https?|wss?|irc):)?(//)?\w+(\.\w+)+/?\S*)', re.UNICODE),
     # bitcoin / data / ed2k / magnet / mail / script / tel, etc.
-    re.compile(r'(?P<url>(bitcoin|data|ed2k|magnet|mailto|script|tel):\w+)', re.ASCII),
-]
-URL_PAT.extend(re.compile(pattern, re.RegexFlag(flags))  # pattern string + compiling flags
-               for pattern, flags in json.loads(os.getenv('DARC_URL_PAT', '[]')))
+    #re.compile(r'(?P<url>(bitcoin|data|ed2k|magnet|mailto|script|tel):\w+)', re.ASCII),
+}
+URL_PAT.update({scheme: re.compile(pattern, re.RegexFlag(flags) | re.ASCII)  # pattern string + compiling flags
+                for scheme, pattern, flags in json.loads(os.getenv('DARC_URL_PAT', '[]'))})
 
 
 def match_proxy(proxy: str) -> bool:
@@ -144,7 +156,7 @@ def match_mime(mime: str) -> bool:
     return MIME_FALLBACK
 
 
-def check_robots(link: 'Link') -> bool:
+def check_robots(link: 'darc_link.Link') -> bool:
     """Check if ``link`` is allowed in ``robots.txt``.
 
     Args:
@@ -172,7 +184,7 @@ def check_robots(link: 'Link') -> bool:
     return True
 
 
-def _check_ng(temp_list: 'List[Link]') -> 'List[Link]':
+def _check_ng(temp_list: 'List[darc_link.Link]') -> 'List[darc_link.Link]':
     """Check content type of links through ``HEAD`` requests.
 
     Args:
@@ -215,10 +227,10 @@ def _check_ng(temp_list: 'List[Link]') -> 'List[Link]':
         except requests.RequestException as error:
             if error.response is None:
                 print(render_error(f'[HEAD] Checking failed <{error}>',
-                                   stem.util.term.Color.RED))  # pylint: disable=no-member
+                                   stem_term.Color.RED))  # pylint: disable=no-member
                 continue
             print(render_error(f'[HEAD] Failed on {error.response.url} <{error}>',
-                               stem.util.term.Color.RED))  # pylint: disable=no-member
+                               stem_term.Color.RED))  # pylint: disable=no-member
             link_list.append(error.response.url)
             continue
         ct_type = get_content_type(response)
@@ -232,7 +244,7 @@ def _check_ng(temp_list: 'List[Link]') -> 'List[Link]':
     return link_list
 
 
-def _check(temp_list: 'List[Link]') -> 'List[Link]':
+def _check(temp_list: 'List[darc_link.Link]') -> 'List[darc_link.Link]':
     """Check hostname and proxy type of links.
 
     Args:
@@ -293,7 +305,7 @@ def get_content_type(response: 'Response') -> str:
     return ct_type.casefold().split(';', maxsplit=1)[0].strip()
 
 
-def extract_links(link: 'Link', html: 'Union[str, bytes]', check: bool = CHECK) -> 'List[Link]':
+def extract_links(link: 'darc_link.Link', html: 'Union[str, bytes]', check: bool = CHECK) -> 'List[darc_link.Link]':
     """Extract links from HTML document.
 
     Args:
@@ -316,7 +328,7 @@ def extract_links(link: 'Link', html: 'Union[str, bytes]', check: bool = CHECK)
     for child in soup.find_all(lambda tag: tag.has_attr('href') or tag.has_attr('src')):
         if (href := child.get('href', child.get('src'))) is None:
             continue
-        temp_link = parse_link(urljoin(link.url, href))
+        temp_link = parse_link(urljoin(link.url, href), backref=link)
         temp_list.append(temp_link)
 
     # extract links from text
@@ -328,7 +340,7 @@ def extract_links(link: 'Link', html: 'Union[str, bytes]', check: bool = CHECK)
     return temp_list
 
 
-def extract_links_from_text(link: 'Link', text: str) -> 'List[Link]':
+def extract_links_from_text(link: 'darc_link.Link', text: str) -> 'List[darc_link.Link]':
     """Extract links from raw text source.
 
     Args:
@@ -352,14 +364,14 @@ def extract_links_from_text(link: 'Link', text: str) -> 'List[Link]':
     """
     temp_list = []
     for part in text.split():
-        for pattern in URL_PAT:
+        for scheme, pattern in URL_PAT.items():
             for match in pattern.finditer(part):
                 match_url = match.group('url')
 
                 # add scheme if not exist
                 if not urlsplit(match_url).scheme:
-                    match_url = f'{link.url_parse.scheme}:{match_url}'
+                    match_url = f'{scheme}:{match_url}'
 
-                temp_link = parse_link(match_url)
+                temp_link = parse_link(match_url, backref=link)
                 temp_list.append(temp_link)
     return temp_list
