@@ -32,7 +32,7 @@ There are three submission events:
 """
 
 import base64
-import dataclasses
+import contextlib
 import glob
 import json
 import os
@@ -43,15 +43,16 @@ import warnings
 from datetime import date
 from typing import TYPE_CHECKING, cast
 
+import peewee
 import requests
-import stem.util.term
+import stem.util.term as stem_term
 
 from darc._compat import datetime
 from darc.const import DEBUG, PATH_DB
 from darc.db import _db_operation
 from darc.error import APIRequestFailed, DatabaseOperaionFailed, render_error
 from darc.model import (HostnameModel, HostsModel, RequestsHistoryModel, RequestsModel, RobotsModel,
-                        SeleniumModel, SitemapModel, URLModel)
+                        SeleniumModel, SitemapModel, URLModel, URLThroughModel)
 from darc.model.utils import Proxy
 from darc.requests import null_session
 
@@ -61,8 +62,8 @@ if TYPE_CHECKING:
     from requests import Response, Session
     from typing_extensions import Literal
 
+    import darc.link as darc_link  # Link
     from darc._typing import File
-    from darc.link import Link
 
     Domain = Literal['new_host', 'requests', 'selenium']
 
@@ -82,39 +83,17 @@ API_REQUESTS = os.getenv('API_REQUESTS')
 API_SELENIUM = os.getenv('API_SELENIUM')
 
 if DEBUG:
-    print(stem.util.term.format('-*- SUBMIT API -*-', stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
-    print(stem.util.term.format(f'NEW HOST: {API_NEW_HOST}', stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
-    print(stem.util.term.format(f'REQUESTS: {API_REQUESTS}', stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
-    print(stem.util.term.format(f'SELENIUM: {API_SELENIUM}', stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
-    print(stem.util.term.format('-' * shutil.get_terminal_size().columns, stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
+    print(stem_term.format('-*- SUBMIT API -*-', stem_term.Color.MAGENTA))  # pylint: disable=no-member
+    print(stem_term.format(f'NEW HOST: {API_NEW_HOST}', stem_term.Color.MAGENTA))  # pylint: disable=no-member
+    print(stem_term.format(f'REQUESTS: {API_REQUESTS}', stem_term.Color.MAGENTA))  # pylint: disable=no-member
+    print(stem_term.format(f'SELENIUM: {API_SELENIUM}', stem_term.Color.MAGENTA))  # pylint: disable=no-member
+    print(stem_term.format('-' * shutil.get_terminal_size().columns, stem_term.Color.MAGENTA))  # pylint: disable=no-member
 
 # UNIX epoch
 EPOCH = datetime(1970, 1, 1, 0, 0)  # 1970-01-01T00:00:00
 
 
-def get_metadata(link: 'Link') -> 'Dict[str, str]':
-    """Generate metadata field.
-
-    Args:
-        link: Link object to generate metadata.
-
-    Returns:
-        The metadata from ``link``.
-
-        * ``url`` -- original URL, :attr:`link.url <darc.link.Link.url>`
-        * ``proxy`` -- proxy type, :attr:`link.proxy <darc.link.Link.proxy>`
-        * ``host`` -- hostname, :attr:`link.host <darc.link.Link.host>`
-        * ``base`` -- base path, :attr:`link.base <darc.link.Link.base>`
-        * ``name`` -- link hash, :attr:`link.name <darc.link.Link.name>`
-
-    """
-    metadata = dataclasses.asdict(link)
-    metadata['base'] = os.path.relpath(link.base, PATH_DB)
-    del metadata['url_parse']
-    return metadata
-
-
-def get_robots(link: 'Link') -> 'Optional[File]':
+def get_robots(link: 'darc_link.Link') -> 'Optional[File]':
     """Read ``robots.txt``.
 
     Args:
@@ -145,7 +124,7 @@ def get_robots(link: 'Link') -> 'Optional[File]':
     }
 
 
-def get_sitemaps(link: 'Link') -> 'Optional[List[File]]':
+def get_sitemaps(link: 'darc_link.Link') -> 'Optional[List[File]]':
     """Read sitemaps.
 
     Args:
@@ -180,7 +159,7 @@ def get_sitemaps(link: 'Link') -> 'Optional[List[File]]':
     return data_list
 
 
-def get_hosts(link: 'Link') -> 'Optional[File]':
+def get_hosts(link: 'darc_link.Link') -> 'Optional[File]':
     """Read ``hosts.txt``.
 
     Args:
@@ -271,11 +250,11 @@ def submit(api: str, domain: 'Domain', data: 'Dict[str, Any]') -> None:
             except requests.RequestException as error:
                 warning = warnings.formatwarning(str(error), APIRequestFailed, __file__, 262,
                                                  f'[{domain.upper()}] response = requests.post(api, json=data)')
-                print(render_error(warning, stem.util.term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
+                print(render_error(warning, stem_term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
     save_submit(domain, data)
 
 
-def submit_new_host(time: 'datetime', link: 'Link', partial: bool = False, force: bool = False) -> None:
+def submit_new_host(time: 'datetime', link: 'darc_link.Link', partial: bool = False, force: bool = False) -> None:
     """Submit new host.
 
     When a new host is discovered, the :mod:`darc` crawler will submit the
@@ -295,7 +274,7 @@ def submit_new_host(time: 'datetime', link: 'Link', partial: bool = False, force
 
     The data submitted should have following format:
 
-    .. code-block:: jsonc
+    .. code-block::
 
         {
             // partial flag - true / false
@@ -317,7 +296,9 @@ def submit_new_host(time: 'datetime', link: 'Link', partial: bool = False, force
                 //   HTML from requests - <base>/<name>_<timestamp>_raw.html
                 //   HTML from selenium - <base>/<name>_<timestamp>.html
                 //   generic data files - <base>/<name>_<timestamp>.dat
-                "name": ...
+                "name": ...,
+                // originate URL - <scheme>://<netloc>/<path>;<params>?<query>#<fragment>
+                "backref": ...
             },
             // requested timestamp in ISO format as in name of saved file
             "Timestamp": ...,
@@ -356,13 +337,12 @@ def submit_new_host(time: 'datetime', link: 'Link', partial: bool = False, force
         * :data:`darc.submit.API_NEW_HOST`
         * :func:`darc.submit.submit`
         * :func:`darc.submit.save_submit`
-        * :func:`darc.submit.get_metadata`
         * :func:`darc.submit.get_robots`
         * :func:`darc.submit.get_sitemaps`
         * :func:`darc.submit.get_hosts`
 
     """
-    metadata = get_metadata(link)
+    metadata = link.asdict()
     ts = time.isoformat()
 
     robots = get_robots(link)
@@ -399,7 +379,7 @@ def submit_new_host(time: 'datetime', link: 'Link', partial: bool = False, force
         except Exception as error:
             warning = warnings.formatwarning(str(error), DatabaseOperaionFailed, __file__, 371,
                                              'submit_new_host(...)')
-            print(render_error(warning, stem.util.term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
+            print(render_error(warning, stem_term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
 
     data = {
         '$PARTIAL$': partial,
@@ -413,11 +393,10 @@ def submit_new_host(time: 'datetime', link: 'Link', partial: bool = False, force
     }
 
     if DEBUG:
-        print(stem.util.term.format('-*- NEW HOST DATA -*-',
-                                    stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
-        print(render_error(pprint.pformat(data), stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
-        print(stem.util.term.format('-' * shutil.get_terminal_size().columns,
-                                    stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
+        print(stem_term.format('-*- NEW HOST DATA -*-',
+                               stem_term.Color.MAGENTA))  # pylint: disable=no-member
+        print(render_error(pprint.pformat(data), stem_term.Color.MAGENTA))  # pylint: disable=no-member
+        print(stem_term.format('-' * shutil.get_terminal_size().columns, stem_term.Color.MAGENTA))  # pylint: disable=no-member
 
     if API_NEW_HOST is None:
         save_submit('new_host', data)
@@ -427,7 +406,7 @@ def submit_new_host(time: 'datetime', link: 'Link', partial: bool = False, force
     submit(API_NEW_HOST, 'new_host', data)
 
 
-def submit_requests(time: 'datetime', link: 'Link',
+def submit_requests(time: 'datetime', link: 'darc_link.Link',
                     response: 'Response', session: 'Session',
                     content: bytes, mime_type: str, html: bool = True) -> None:
     """Submit requests data.
@@ -450,7 +429,7 @@ def submit_requests(time: 'datetime', link: 'Link',
 
     The data submitted should have following format:
 
-    .. code-block:: jsonc
+    .. code-block::
 
         {
             // metadata of URL
@@ -468,7 +447,9 @@ def submit_requests(time: 'datetime', link: 'Link',
                 //   HTML from requests - <base>/<name>_<timestamp>_raw.html
                 //   HTML from selenium - <base>/<name>_<timestamp>.html
                 //   generic data files - <base>/<name>_<timestamp>.dat
-                "name": ...
+                "name": ...,
+                // originate URL - <scheme>://<netloc>/<path>;<params>?<query>#<fragment>
+                "backref": ...
             },
             // requested timestamp in ISO format as in name of saved file
             "Timestamp": ...,
@@ -519,7 +500,6 @@ def submit_requests(time: 'datetime', link: 'Link',
         * :data:`darc.submit.API_REQUESTS`
         * :func:`darc.submit.submit`
         * :func:`darc.submit.save_submit`
-        * :func:`darc.submit.get_metadata`
         * :func:`darc.submit.get_raw`
         * :func:`darc.crawl.crawler`
 
@@ -556,6 +536,12 @@ def submit_requests(time: 'datetime', link: 'Link',
                 url.last_seen = time
             _db_operation(url.save)
 
+            if link.url_backref is not None:
+                with contextlib.suppress(peewee.IntegrityError):
+                    _db_operation(URLThroughModel.create,
+                                  parent=_db_operation(URLModel.get_by_url, link.url_backref.url),
+                                  child=url)
+
             model = cast('RequestsModel',
                          _db_operation(RequestsModel.create,
                                        url=url,
@@ -587,9 +573,9 @@ def submit_requests(time: 'datetime', link: 'Link',
         except Exception as error:
             warning = warnings.formatwarning(str(error), DatabaseOperaionFailed, __file__, 528,
                                              'submit_requests(...)')
-            print(render_error(warning, stem.util.term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
+            print(render_error(warning, stem_term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
 
-    metadata = get_metadata(link)
+    metadata = link.asdict()
     ts = time.isoformat()
 
     if html:
@@ -626,11 +612,10 @@ def submit_requests(time: 'datetime', link: 'Link',
     }
 
     if DEBUG:
-        print(stem.util.term.format('-*- REQUESTS DATA -*-',
-                                    stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
-        print(render_error(pprint.pformat(data), stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
-        print(stem.util.term.format('-' * shutil.get_terminal_size().columns,
-                                    stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
+        print(stem_term.format('-*- REQUESTS DATA -*-',
+                               stem_term.Color.MAGENTA))  # pylint: disable=no-member
+        print(render_error(pprint.pformat(data), stem_term.Color.MAGENTA))  # pylint: disable=no-member
+        print(stem_term.format('-' * shutil.get_terminal_size().columns, stem_term.Color.MAGENTA))  # pylint: disable=no-member
 
     if API_REQUESTS is None:
         save_submit('requests', data)
@@ -640,7 +625,7 @@ def submit_requests(time: 'datetime', link: 'Link',
     submit(API_REQUESTS, 'requests', data)
 
 
-def submit_selenium(time: 'datetime', link: 'Link',
+def submit_selenium(time: 'datetime', link: 'darc_link.Link',
                     html: str, screenshot: 'Optional[str]') -> None:
     """Submit selenium data.
 
@@ -664,7 +649,7 @@ def submit_selenium(time: 'datetime', link: 'Link',
 
     The data submitted should have following format:
 
-    .. code-block:: jsonc
+    .. code-block::
 
         {
             // metadata of URL
@@ -682,7 +667,9 @@ def submit_selenium(time: 'datetime', link: 'Link',
                 //   HTML from requests - <base>/<name>_<timestamp>_raw.html
                 //   HTML from selenium - <base>/<name>_<timestamp>.html
                 //   generic data files - <base>/<name>_<timestamp>.dat
-                "name": ...
+                "name": ...,
+                // originate URL - <scheme>://<netloc>/<path>;<params>?<query>#<fragment>
+                "backref": ...
             },
             // requested timestamp in ISO format as in name of saved file
             "Timestamp": ...,
@@ -710,7 +697,6 @@ def submit_selenium(time: 'datetime', link: 'Link',
         * :data:`darc.submit.API_SELENIUM`
         * :func:`darc.submit.submit`
         * :func:`darc.submit.save_submit`
-        * :func:`darc.submit.get_metadata`
         * :func:`darc.submit.get_html`
         * :func:`darc.submit.get_screenshot`
         * :func:`darc.crawl.loader`
@@ -745,6 +731,12 @@ def submit_selenium(time: 'datetime', link: 'Link',
                 url.last_seen = time
             _db_operation(url.save)
 
+            if link.url_backref is not None:
+                with contextlib.suppress(peewee.IntegrityError):
+                    _db_operation(URLThroughModel.create,
+                                  parent=_db_operation(URLModel.get_by_url, link.url_backref.url),
+                                  child=url)
+
             _db_operation(SeleniumModel.create,
                           url=url,
                           timestamp=time,
@@ -753,9 +745,9 @@ def submit_selenium(time: 'datetime', link: 'Link',
         except Exception as error:
             warning = warnings.formatwarning(str(error), DatabaseOperaionFailed, __file__, 720,
                                              'submit_selenium(...)')
-            print(render_error(warning, stem.util.term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
+            print(render_error(warning, stem_term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
 
-    metadata = get_metadata(link)
+    metadata = link.asdict()
     ts = time.isoformat()
 
     if screenshot is None:
@@ -778,11 +770,10 @@ def submit_selenium(time: 'datetime', link: 'Link',
     }
 
     if DEBUG:
-        print(stem.util.term.format('-*- SELENIUM DATA -*-',
-                                    stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
-        print(render_error(pprint.pformat(data), stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
-        print(stem.util.term.format('-' * shutil.get_terminal_size().columns,
-                                    stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
+        print(stem_term.format('-*- SELENIUM DATA -*-',
+                               stem_term.Color.MAGENTA))  # pylint: disable=no-member
+        print(render_error(pprint.pformat(data), stem_term.Color.MAGENTA))  # pylint: disable=no-member
+        print(stem_term.format('-' * shutil.get_terminal_size().columns, stem_term.Color.MAGENTA))  # pylint: disable=no-member
 
     if API_SELENIUM is None:
         save_submit('selenium', data)
